"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var rxjs_1 = require("rxjs");
var metadata_1 = require("./metadata");
var EmitterMetadata;
(function (EmitterMetadata) {
    var ProxyMode;
    (function (ProxyMode) {
        ProxyMode.None = "None";
        ProxyMode.From = "From";
        ProxyMode.Alias = "Alias";
        ProxyMode.Merge = "Merge";
    })(ProxyMode = EmitterMetadata.ProxyMode || (EmitterMetadata.ProxyMode = {}));
    var SubjectInfo;
    (function (SubjectInfo) {
        function IsDynamicAlias(subjectInfo) {
            return !IsStaticAlias(subjectInfo);
        }
        SubjectInfo.IsDynamicAlias = IsDynamicAlias;
        function IsStaticAlias(subjectInfo) {
            return (subjectInfo.observable instanceof rxjs_1.Subject);
        }
        SubjectInfo.IsStaticAlias = IsStaticAlias;
    })(SubjectInfo = EmitterMetadata.SubjectInfo || (EmitterMetadata.SubjectInfo = {}));
    EmitterMetadata.EmitterMapSymbol = Symbol("EmitterMapSymbol");
    /** @description Gets the metadata map object for the given target class (or its inheritted classes). */
    function GetMetadataMap(target) {
        return metadata_1.Metadata.GetMetadata(EmitterMetadata.EmitterMapSymbol, target, new Map());
    }
    EmitterMetadata.GetMetadataMap = GetMetadataMap;
    /** @description Gets the metadata map object for the given target class. */
    function GetOwnMetadataMap(target) {
        return metadata_1.Metadata.GetOwnMetadata(EmitterMetadata.EmitterMapSymbol, target, new Map());
    }
    EmitterMetadata.GetOwnMetadataMap = GetOwnMetadataMap;
    function HasOwnMetadataMap(target) {
        return Reflect.hasOwnMetadata(EmitterMetadata.EmitterMapSymbol, target);
    }
    EmitterMetadata.HasOwnMetadataMap = HasOwnMetadataMap;
    function SetMetadataMap(target, map) {
        metadata_1.Metadata.SetMetadata(EmitterMetadata.EmitterMapSymbol, target, map);
    }
    EmitterMetadata.SetMetadataMap = SetMetadataMap;
    /** @description Copy all metadata from the source map to the target map.
     *
     *  Note: This mutates the target map.
     **/
    function CopyMetadata(target, source, overwrite) {
        // Iterate over all source metadata properties...
        source.forEach(function (subjectInfo, eventType) {
            // And add them to this class' metadata map if not already defined
            if (overwrite || !target.has(eventType)) {
                target.set(eventType, Object.assign({}, subjectInfo));
            }
        });
        return target;
    }
    EmitterMetadata.CopyMetadata = CopyMetadata;
    /** @description Merge own and inheritted metadata into a single map.
     *
     *  Note: This mutates the object's metadata.
     **/
    function CopyInherittedMetadata(object) {
        if (object) {
            var metadataMap = GetMetadataMap(object);
            var inherittedMap = CopyInherittedMetadata(Object.getPrototypeOf(object));
            return CopyMetadata(metadataMap, inherittedMap);
        }
        return new Map();
    }
    EmitterMetadata.CopyInherittedMetadata = CopyInherittedMetadata;
})(EmitterMetadata = exports.EmitterMetadata || (exports.EmitterMetadata = {}));
