"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var metadata_1 = require("./metadata");
var EventMetadata;
(function (EventMetadata) {
    EventMetadata.EventMapSymbol = Symbol("EventMapSymbol");
    /** @description Gets the metadata map object for the given target class (or its inheritted classes). */
    function GetMetadataMap(target) {
        return metadata_1.Metadata.GetMetadata(EventMetadata.EventMapSymbol, target, new Map());
    }
    EventMetadata.GetMetadataMap = GetMetadataMap;
    /** @description Gets the metadata map object for the given target class. */
    function GetOwnMetadataMap(target) {
        return metadata_1.Metadata.GetOwnMetadata(EventMetadata.EventMapSymbol, target, new Map());
    }
    EventMetadata.GetOwnMetadataMap = GetOwnMetadataMap;
    /** @description
     *  Gets the property subject map for the given event type from the metadata map for the given target class (or its inheritted classes).
     */
    function GetPropertySubjectMap(type, target) {
        var map = GetMetadataMap(target);
        var subjectMap = map.get(type);
        if (!subjectMap) {
            subjectMap = new Map();
            map.set(type, subjectMap);
        }
        return subjectMap;
    }
    EventMetadata.GetPropertySubjectMap = GetPropertySubjectMap;
    /** @description
     *  Gets the property subject map for the given event type from the metadata map for the given target class.
     */
    function GetOwnPropertySubjectMap(type, target) {
        var map = GetOwnMetadataMap(target);
        var subjectMap = map.get(type);
        if (!subjectMap) {
            subjectMap = new Map();
            map.set(type, subjectMap);
        }
        return subjectMap;
    }
    EventMetadata.GetOwnPropertySubjectMap = GetOwnPropertySubjectMap;
    function HasOwnMetadataMap(target) {
        return Reflect.hasOwnMetadata(EventMetadata.EventMapSymbol, target);
    }
    EventMetadata.HasOwnMetadataMap = HasOwnMetadataMap;
    function SetMetadataMap(target, map) {
        metadata_1.Metadata.SetMetadata(EventMetadata.EventMapSymbol, target, map);
    }
    EventMetadata.SetMetadataMap = SetMetadataMap;
    /** @description Copy all metadata from the source map to the target map.
     *
     *  Note: This mutates the target map.
     **/
    function CopyMetadata(target, source, overwrite) {
        // Iterate over all source metadata properties...
        source.forEach(function (propertySubjectMap, eventType) { return propertySubjectMap.forEach(function (value, propertyKey) {
            var targetPropertySubjectMap;
            // Get the property subject map (or create it if it doesn't exist for this eventType)
            if (target.has(eventType)) {
                targetPropertySubjectMap = target.get(eventType);
            }
            else {
                targetPropertySubjectMap = new Map();
                target.set(eventType, targetPropertySubjectMap);
            }
            // And add them to this class' metadata map if not already defined
            if (overwrite || !targetPropertySubjectMap.has(propertyKey)) {
                targetPropertySubjectMap.set(propertyKey, Object.assign({}, value));
            }
        }); });
        return target;
    }
    EventMetadata.CopyMetadata = CopyMetadata;
    /** @description Merge own and inheritted metadata into a single map.
     *
     *  Note: This mutates the object's metadata.
     **/
    function CopyInherittedMetadata(object) {
        if (object) {
            var metadataMap = GetMetadataMap(object);
            var inherittedMap = CopyInherittedMetadata(Object.getPrototypeOf(object));
            // Merge own and inheritted metadata into a single map (note: this mutates object's metadata)
            return CopyMetadata(metadataMap, inherittedMap);
        }
        return new Map();
    }
    EventMetadata.CopyInherittedMetadata = CopyInherittedMetadata;
})(EventMetadata = exports.EventMetadata || (exports.EventMetadata = {}));
