"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var rxjs_1 = require("rxjs");
var emitter_metadata_1 = require("./emitter-metadata");
var observable_util_1 = require("./observable-util");
/** @PropertyDecoratorFactory */
function StateEmitter() {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    var paramsArg;
    if (args.length > 0) {
        paramsArg = args[0];
    }
    if (!paramsArg || paramsArg instanceof Function) {
        return StateEmitter.WithParams.apply(StateEmitter, [undefined].concat(args));
    }
    else {
        return StateEmitter.WithParams.apply(StateEmitter, [paramsArg].concat(args.slice(1)));
    }
}
exports.StateEmitter = StateEmitter;
(function (StateEmitter) {
    /** @PropertyDecoratorFactory */
    function WithParams(params) {
        var propertyDecorators = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            propertyDecorators[_i - 1] = arguments[_i];
        }
        params = params || {};
        /** @PropertyDecorator */
        return function $$StateEmitterDecorator(target, propertyKey) {
            // If an emitterType wasn't specified...
            if (!params.propertyName) {
                // Try to deduce the emitterType from the propertyKey
                if (propertyKey.endsWith("$")) {
                    params.propertyName = propertyKey.substring(0, propertyKey.length - 1);
                }
                else {
                    throw new Error("@StateEmitter error: emitterType could not be deduced from propertyKey \"" + propertyKey + "\" (only keys ending with '$' can be auto-deduced).");
                }
            }
            // Set the emitter type of the decorator
            $$StateEmitterDecorator["emitterType"] = params.propertyName;
            // Apply any property decorators to the property
            propertyDecorators.forEach(function (propertyDecorator) { return propertyDecorator(target, params.propertyName); });
            // Create the event source metadata for the decorated property
            StateEmitter.CreateMetadata(target, params.propertyName, Object.assign({ propertyKey: propertyKey, observable: undefined }, params));
        };
    }
    StateEmitter.WithParams = WithParams;
    //# Helper Decorators
    /////////////////////////////
    function _ResolveProxyDecoratorParams(params) {
        return typeof params === "string" ? { path: params } : params;
    }
    StateEmitter._ResolveProxyDecoratorParams = _ResolveProxyDecoratorParams;
    /** @PropertyDecoratorFactory */
    function Alias(params) {
        var propertyDecorators = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            propertyDecorators[_i - 1] = arguments[_i];
        }
        var $params = _ResolveProxyDecoratorParams(params);
        return WithParams.apply(void 0, [{
                propertyName: $params.propertyName,
                proxyMode: emitter_metadata_1.EmitterMetadata.ProxyMode.Alias,
                proxyPath: $params.path,
                proxyMergeUpdates: $params.mergeUpdates
            }].concat(propertyDecorators));
    }
    StateEmitter.Alias = Alias;
    /** @PropertyDecoratorFactory */
    function From(params) {
        var propertyDecorators = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            propertyDecorators[_i - 1] = arguments[_i];
        }
        var $params = _ResolveProxyDecoratorParams(params);
        return WithParams.apply(void 0, [{
                propertyName: $params.propertyName,
                proxyMode: emitter_metadata_1.EmitterMetadata.ProxyMode.From,
                proxyPath: $params.path,
                proxyMergeUpdates: $params.mergeUpdates
            }].concat(propertyDecorators));
    }
    StateEmitter.From = From;
    /** @PropertyDecoratorFactory */
    function Merge(params) {
        var propertyDecorators = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            propertyDecorators[_i - 1] = arguments[_i];
        }
        var $params = _ResolveProxyDecoratorParams(params);
        return WithParams.apply(void 0, [{
                propertyName: $params.propertyName,
                proxyMode: emitter_metadata_1.EmitterMetadata.ProxyMode.Merge,
                proxyPath: $params.path,
                proxyMergeUpdates: $params.mergeUpdates
            }].concat(propertyDecorators));
    }
    StateEmitter.Merge = Merge;
    var Facade;
    (function (Facade) {
        function CreateSetter(type) {
            return function (value) {
                var subjectInfo = emitter_metadata_1.EmitterMetadata.GetMetadataMap(this).get(type);
                // If this is a static subject...
                if (emitter_metadata_1.EmitterMetadata.SubjectInfo.IsStaticAlias(subjectInfo)) {
                    // Notify the subject of the new value
                    subjectInfo.observable.next(value);
                }
                else {
                    try {
                        // If value merging wasn't explicitly enabled or disabled then only enable it if the incoming value is an Object
                        var mergeUpdates = subjectInfo.proxyMergeUpdates;
                        mergeUpdates = (mergeUpdates !== null && mergeUpdates !== undefined) ? mergeUpdates : value instanceof Object;
                        // Update the dynamic proxy value
                        observable_util_1.ObservableUtil.UpdateDynamicPropertyPathValue(this, subjectInfo.proxyPath, value, mergeUpdates);
                    }
                    catch (e) {
                        console.error("Unable to set value for proxy StateEmitter \"" + this.constructor.name + "." + type + "\" with dynamic property path \"" + subjectInfo.proxyPath + "\" - Path does not contain a Subject.");
                    }
                }
            };
        }
        Facade.CreateSetter = CreateSetter;
        function CreateGetter(type, initialValue) {
            var lastValue = initialValue;
            var subscription;
            return function () {
                // Create a subscription to subject changes if not subscribed
                if (!subscription) {
                    // Update when a new value is emitted
                    var subjectInfo = emitter_metadata_1.EmitterMetadata.GetMetadataMap(this).get(type);
                    subscription = subjectInfo.observable.subscribe(function (value) { return lastValue = value; });
                }
                // Return the last value that was emitted
                return lastValue;
            };
        }
        Facade.CreateGetter = CreateGetter;
    })(Facade = StateEmitter.Facade || (StateEmitter.Facade = {}));
    function Bootstrap(targetInstance) {
        function DefineProxyObservableGetter(subjectInfo, alwaysResolvePath, onResolve) {
            var observable;
            // Create a getter that resolves the observable from the target proxy path
            Object.defineProperty(subjectInfo, "observable", { get: function () {
                    if (alwaysResolvePath || !observable) {
                        // Get the proxy observable
                        observable = observable_util_1.ObservableUtil.ResolvePropertyPath(targetInstance, subjectInfo.proxyPath);
                        if (onResolve) {
                            observable = onResolve(observable) || observable;
                        }
                    }
                    return observable;
                } });
        }
        // Copy all emitter metadata from the constructor to the target instance
        var metadataMap = emitter_metadata_1.EmitterMetadata.CopyMetadata(emitter_metadata_1.EmitterMetadata.GetOwnMetadataMap(targetInstance), emitter_metadata_1.EmitterMetadata.CopyInherittedMetadata(targetInstance.constructor), true);
        // Iterate over each of the target properties for each emitter type used in this class
        metadataMap.forEach(function (subjectInfo, emitterType) {
            // Check the proxy mode for this subject
            switch (subjectInfo.proxyMode) {
                // Aliased emitters simply pass directly through to their source value
                case emitter_metadata_1.EmitterMetadata.ProxyMode.Alias: {
                    DefineProxyObservableGetter(subjectInfo, true);
                    break;
                }
                // Merge emitters create a separate subject that subscribes to all emissions from the source
                // From emitters create a separate subject that sets the initial value from the source
                case emitter_metadata_1.EmitterMetadata.ProxyMode.Merge:
                case emitter_metadata_1.EmitterMetadata.ProxyMode.From: {
                    // Create a new copy subject
                    var subject_1 = new rxjs_1.BehaviorSubject(subjectInfo.initialValue);
                    // Create a getter that returns the new subject
                    DefineProxyObservableGetter(subjectInfo, false, function (proxyObservable) {
                        // Only take the first value if this is a From proxy
                        if (subjectInfo.proxyMode === emitter_metadata_1.EmitterMetadata.ProxyMode.From) {
                            proxyObservable = proxyObservable.take(1);
                        }
                        proxyObservable.subscribe(function (value) { return subject_1.next(value); });
                        return subject_1;
                    });
                    break;
                }
                case emitter_metadata_1.EmitterMetadata.ProxyMode.None:
                default: {
                    // Create a new BehaviorSubject with the default value
                    subjectInfo.observable = new rxjs_1.BehaviorSubject(subjectInfo.initialValue);
                    break;
                }
            }
            // Assign the facade getter and setter to the target instance for this EmitterType
            Object.defineProperty(targetInstance, emitterType, {
                get: Facade.CreateGetter(emitterType, subjectInfo.initialValue),
                set: subjectInfo.readOnly ? undefined : Facade.CreateSetter(emitterType)
            });
            // Create a getter property on the targetInstance that lazily retreives the observable
            Object.defineProperty(targetInstance, subjectInfo.propertyKey, { get: function () { return subjectInfo.observable; } });
        });
    }
    StateEmitter.Bootstrap = Bootstrap;
    function CreateMetadata(target, type, metadata) {
        if (target[type]) {
            // Make sure the target class doesn't have a custom property already defined for this event type
            throw new Error("@StateEmitter metadata creation failed. Class already has a custom " + type + " property.");
        }
        // Add the propertyKey to the class' metadata
        emitter_metadata_1.EmitterMetadata.GetOwnMetadataMap(target.constructor).set(type, metadata);
    }
    StateEmitter.CreateMetadata = CreateMetadata;
})(StateEmitter = exports.StateEmitter || (exports.StateEmitter = {}));
